"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.cleanBodyData = void 0;
const Util_1 = require("./Util");
const DEFAULT_TIME_LIMIT = 60000000; //One Minute;
const DEFAULT_SIZE_LIMIT = 500000000; //500MB
/** Clean Body Data
 *
 * @param {RawBodyData} data
 * @returns {Promise<BodyData>}
 */
async function cleanBodyData(data) {
    if (data instanceof Error)
        return data;
    for (let [key, value] of data) {
        if (value instanceof Blob) {
            data.set(key, await value.text());
        }
    }
    //@ts-ignore
    return data;
}
exports.cleanBodyData = cleanBodyData;
/** Parse Headers from String
 *
 * @param {string} value
 */
function parseHeaders(value) {
    const headers = {};
    for (const header of value.split(/;|\n/g)) {
        if (header.trim() !== "") {
            let [name, value] = header.split(/:|=/g);
            if (value === undefined) {
                value = "";
            }
            else {
                value = value.trim();
            }
            if (value.indexOf('"') >= 0) {
                headers[name.trim()] = value.substring(1, value.length - 1);
            }
            else {
                headers[name.trim()] = value;
            }
        }
    }
    return headers;
}
/** Process Cloudflare Request
 *
 * @param {Request} request
 * @returns {Promise<RawBodyData>}
 */
function processCloudflareRequest(request) {
    return new Promise((res, rej) => {
        const content = request.headers.get("Content-Type")?.toLocaleLowerCase();
        if (content === undefined || !content.includes("form-data")) {
            return res(new Map());
        }
        request.formData().then((data) => {
            const map = new Map();
            data.forEach((value, key) => {
                map.set(key, value);
            });
            res(map);
        }).catch(rej);
    });
}
/** Process Node Request
 *
 * @param {IncomingMessage} request
 * @returns {Promise<RawBodyData>}
 */
function processNodeRequest(request) {
    let multipart = false;
    let boundry = new RegExp("&");
    const type = request.headers["content-type"]?.toLocaleLowerCase();
    if (type && type.indexOf("form-data") >= 0) {
        const header = parseHeaders(type);
        if (header["boundary"] === undefined)
            throw new Error("Header is malformed.");
        boundry = new RegExp(header["boundary"].replace(/-+/, "-+"));
        multipart = true;
    }
    let buffer = "";
    let output = new Map();
    /** Process Multipart Data
     *
     * @param {string} data
     * @returns {name:string, info:FileData|string}
     */
    const processMultipartData = (data) => {
        //Multipart Headers Match
        const match = data.match(/[\d\D\n]*?\r\n\r\n/);
        if (match === null)
            throw new Error("Malformed Multipart Data!");
        //Seperate Header from data/info
        const headers = parseHeaders(match[0]);
        data = data.substring(match[0].length);
        //Make sure name is found.
        if (headers["name"] === undefined)
            throw new Error("No name given in multipart data!");
        //Detect if File
        if (headers["filename"] !== undefined) {
            [
                headers["name"],
                new Blob([data], { type: "text/plain" })
            ];
        }
        return [
            headers["name"],
            data
        ];
    };
    /** Takes data as a string and process the information.
     *
     * @param {string} data
     */
    const processData = (data) => {
        const [name, value] = multipart ? processMultipartData(data) : data.split("=");
        output.set(name, value);
    };
    /** Process Buffer
     *
     * Loops through the buffer untill all boundrys are crossed.
     */
    const processBuffer = () => {
        let match = buffer.match(boundry);
        while (match !== null) {
            const index = buffer.indexOf(match[0]);
            if (index === 0) {
                buffer = buffer.substring(match[0].length);
            }
            else {
                processData(buffer.slice(0, index));
                buffer = buffer.slice(index + match[0].length);
            }
            match = buffer.match(boundry);
        }
    };
    return new Promise((resolve) => {
        /** Reject Replacement that instead resolves the error.
         *
         * @param {any} e
         */
        const reject = (e) => {
            if (!(e instanceof Error)) {
                e = new Error(String(e));
            }
            resolve(e);
        };
        request.on("data", chunk => {
            try {
                buffer += String(chunk);
                if (buffer.length >= DEFAULT_SIZE_LIMIT)
                    throw new Error("Upload size limit reached!");
                processBuffer();
            }
            catch (err) {
                reject(err);
            }
        });
        request.on("error", reject);
        request.on("end", () => {
            resolve(output);
        });
        setTimeout(() => {
            reject(new Error("Upload timeout limit reached!"));
        }, DEFAULT_TIME_LIMIT);
    });
}
async function BodyParser(request) {
    return cleanBodyData((0, Util_1.isCloudflareRequest)(request)
        ? (await processCloudflareRequest(request))
        : (await processNodeRequest(request)));
}
exports.default = BodyParser;
