"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const OutgoingResponse_1 = __importDefault(require("./OutgoingResponse"));
const IncomingRequest_1 = __importDefault(require("./IncomingRequest"));
const MimeTypes_1 = __importDefault(require("../MimeTypes"));
const Util_1 = require("../Util");
const HTML_MIME_TYPE = (0, MimeTypes_1.default)("html");
const TXT_MIME_TYPE = (0, MimeTypes_1.default)("txt");
const JSON_MIME_TYPE = (0, MimeTypes_1.default)("json");
/** Context
 *
 * Wrapper Around Request/Response
 */
class Context {
    /** Constructor
     *
     * @param request
     * @param response
     */
    constructor(request, response, data, env, view, auth) {
        this._request = new IncomingRequest_1.default(request);
        this._response = new OutgoingResponse_1.default(response);
        this._env = env;
        this._url = (0, Util_1.inCloudfareWorker)()
            ? new URL(this._request.url)
            : new URL(request.url || "/", `http://${this._request.headers.get("host")}`);
        this._view = view;
        this._auth = auth;
        //Defaults
        this._search = new Map();
        this._params = new Map();
        this._form = data;
        //Search Values
        for (const [name, value] of this._url.searchParams.entries())
            this._search.set(name, value);
    }
    /** Request Getter
     *
     */
    get request() {
        return this._request;
    }
    /** Response Getter
     *
     */
    get response() {
        return this._response;
    }
    /** Form Data Getter
     *
     */
    get formData() {
        return this._form;
    }
    /** Environment Getter
     *
     */
    get env() {
        return this._env;
    }
    /** Url Getter
     *
     */
    get url() {
        return this._url;
    }
    /** Method Getter
     *
     */
    get method() {
        return this._request.method;
    }
    /** Params Getter
     *
     */
    get params() {
        return this._params;
    }
    /** Search Getter
     *
     */
    get search() {
        return this._search;
    }
    /** Set Status Code
     *
     * @param {number} value
     * @returns {this}
     */
    status(value) {
        if (typeof value !== "number") {
            value = Number(value);
        }
        if (isNaN(value)) {
            throw new TypeError("Status code must be a nubmer!");
        }
        else if (value < 100 || value > 999) {
            throw new TypeError("Status code is out of range!");
        }
        this._response.status = value;
        return this;
    }
    /** Write Chunk to Response
     *
     * @param {any} chunk
     * @returns {this}
     */
    write(chunk) {
        this._response.write(chunk);
        return this;
    }
    /** End Response
     *
     */
    end() {
        this._response.end();
    }
    /** Set Json Content
     *
     * @param {Object} object
     * @returns {this}
     */
    json(object) {
        if (!this._response.headers.get("Content-Type")) {
            this._response.headers.set('Content-Type', JSON_MIME_TYPE);
        }
        this._response.write(JSON.stringify(object));
        this._response.end();
        return this;
    }
    /** Set Text Content
     *
     * @param {string} value
     * @returns {this}
     */
    text(value) {
        if (!this._response.headers.get("Content-Type")) {
            this._response.headers.set('Content-Type', TXT_MIME_TYPE);
        }
        this._response.write(value);
        this._response.end();
        return this;
    }
    /** Set HTML Content
     *
     * @param {string} value
     * @returns {this}
     */
    html(value) {
        if (!this._response.headers.get("Content-Type")) {
            this._response.headers.set('Content-Type', HTML_MIME_TYPE);
        }
        this._response.write(value);
        this._response.end();
        return this;
    }
    /** Expects Render
     *
     * Checks headers to test if request is expecting render object.
     *
     * @returns {boolean}
     */
    expectsRender() {
        return this._request.headers.get(Util_1.HEADER_KEY) === Util_1.HEADER_VALUE;
    }
    /** Reunder Update Content
     *
     * @param {ContentUpdate} value
     */
    render(value) {
        if (this._view === undefined)
            throw new Error("No view to render with!");
        this._response.headers.set(Util_1.HEADER_KEY, Util_1.HEADER_VALUE);
        if (this.expectsRender()) {
            this.json(this._view.update(value));
        }
        else {
            this.html(this._view.render(value));
        }
    }
    /** Flush Update Content
     *
     */
    flush() {
        return this._response.flush();
    }
    /** Get Authorization
     *
     * @returns {Promise<User|null>}
     */
    async getAuth() {
        if (this._auth === undefined || this._auth.get() === undefined)
            return null;
        return await this._auth.get()(this._request);
    }
    async setAuth(user) {
        if (this._auth === undefined || this._auth.set() === undefined)
            return;
        await this._auth.set()(this._response, user);
    }
    /** Redirect
     *
     * @param url
     * @param status
     */
    redirect(url, status) {
        if (url === "back") {
            url = this._request.headers.get("Referrer") || "/";
        }
        if (this._request.headers.get(Util_1.HEADER_KEY) === Util_1.HEADER_VALUE) {
            if (url instanceof URL) {
                url = url.pathname;
            }
            this.render({
                redirect: url
            });
        }
        else {
            if (typeof url === "string") {
                url = new URL(this._request.url.replace(/(^https?:\/\/.*?:\d{0,4})(.*)$/, "$1" + url));
            }
            this._response.redirect(url, status);
        }
    }
}
exports.default = Context;
